# -*- coding: utf-8 -*-
#
#
#==============================================================================
# Objectives of this PythonXT for Imaris:
#   Segments nucleus, nucleolus and chromocenters into surfaces in DAPI channel,
#   Segments FISH signal into spots in FISH channels
#   Calculate distance of each FISH signal to nucleus surface, to nucleolus and to the chromocenters
#   Exports tables containing calculated distance and one table containing the coordinates of the spots
# Note: This script is calibrated for single copy gene FISH on plant nuclei
# Creator: Mariamawit S. Ashenafi, Célia Baroux, UZH
# Published on 23.01.2016
#==============================================================================
#
#
#    <CustomTools>
#      <Menu>
#       <Item name="FISHInsideNucleus" icon="Python" tooltip="FISHInsideNucleus">
#         <Command>PythonXT::XTFISHInsideNucleus(%i)</Command>
#       </Item>
#      </Menu>
#    </CustomTools>

import numpy as np
import pandas as pd
import logging
import Tkinter as tk
import tkMessageBox
import os
from Tkinter import *
import Tkinter, tkFileDialog
import time 
import datetime 
import ImarisLib
import ttk
#==============================================================================
# Start of extension
#==============================================================================

#==============================================================================
# Functions required to log and display progress of the plugin
#==============================================================================
def logtime(aTitle):
  # type: (object) -> object
  global gLasttime
  curtime = datetime.datetime.now()
  if (gLasttime is not None):
    diff = (curtime-gLasttime).total_seconds()
  else:
    diff = '-'
  gLasttime = curtime
  print curtime.ctime(), '[', str(diff), ']', aTitle

#==============================================================================
# Functions required to create new channel to improve segmentation
#==============================================================================

#This function returns the maximum value in a dataframe  
def getMax(ListArgument):
    maxList         =   [max(i) for j in ListArgument for i in j]
    maxValue        =   max(maxList)
    return maxValue

#This function returns the image in the form of float value, and get surface table (inside table value=1, outside table value=0)
def Get_Mask_data(Surf, ch):
    vImage			=	vImaris.GetDataSet()
    vImageSizeX 	=	vImage.GetSizeX()
    vImageSizeY 	=	vImage.GetSizeY()
    vImageSizeZ 	=	vImage.GetSizeZ()
    vExtentMinX		=	vImage.GetExtendMinX()
    vExtentMinY		=	vImage.GetExtendMinY()
    vExtentMinZ		=	vImage.GetExtendMinZ()
    vExtentMaxX		=	vImage.GetExtendMaxX()
    vExtentMaxY		=	vImage.GetExtendMaxY()
    vExtentMaxZ		=	vImage.GetExtendMaxZ()
    mask_min		=	[vExtentMinX, vExtentMinY, vExtentMinZ]
    mask_max		=	[vExtentMaxX, vExtentMaxY, vExtentMaxZ]
    mask_size		=	[vImageSizeX, vImageSizeY, vImageSizeZ]
    mask_time		=	0
    mask			=	Surf.GetMask(mask_min[0], mask_min[1], mask_min[2], mask_max[0], mask_max[1], mask_max[2],mask_size[0],mask_size[1], mask_size[2], mask_time)
    mask_values		=	mask.GetDataVolumeFloats(0,0)	
    vImage_data     =   vImage.GetDataVolumeFloats(ch,0)
    return mask_values, vImage_data

#This function, inside a given surface, selectes voxels containing 50% of the highest intensity, sets there value to the maximum intensity
def getHightIntensity(mask_values, vImage_data):
    maxValue=getMax(vImage_data)
    Result                  =   []
    for z in range(len(vImage_data)):
        maskY               =   []
        for y in range(len(vImage_data[0])):
            mask_surface	=	mask_values[z][y]
            mask            =	vImage_data[z][y]
            mask            =   [round(item/maxValue, 0) for item in mask]
            mask            =	[maxValue if item==1 else item for item in mask]
            mask            =	[x*y for x,y in zip(mask, mask_surface)]
            maskY.append(mask)
        Result.append(maskY)
    return Result
 
#This function adds a channel in the image, and set the data for this channel   
def AddChannel(table, vImage, numberIndex):
    vImage.SetSizeC(numberIndex)     
    vImage.SetDataVolumeFloats(table, numberIndex-1, 0)

#==============================================================================
# Functions required to segment DAPI and FISH channels
#==============================================================================
#Segments a channel into surface and returns the surface
def Segment_Surface(vImage, ch, vSFW, name,vLCF):
    vROI = None
    vATA = 1
    vATM = 0
    vSFS = ''
    vDCI = ch
    vSurface2 = vImaris.GetImageProcessing().DetectSurfaces(vImage, vROI, vDCI,vSFW, vLCF, vATA, vATM, vSFS)
    vSurface2.SetName(name)
    return vSurface2
    
#This function, inside a given surface, selectes voxels containing 50% of the lowest intensity, sets there value to the maximum intensity, and sets the value of the other voxels to 0      
def getLowIntensity(mask_values, vImage_data, mask_values2):
    maxValue                =   getMax(vImage_data)
    Result                  =   []
    for z in range(len(vImage_data)):
        mask_y              =   []
        for y in range(len(vImage_data[0])):
            mask_surface	=	mask_values[z][y]
            mask_surface2	=	mask_values2[z][y]
            mask            =	vImage_data[z][y]
            # mask=[1 for item==0 else item for item in mask]
            mask            =	[round(item/maxValue, 1) for item in mask]
            mask            =   [0 if item>=0.5 else item for item in mask]
            mask_surface    =   [maxValue if item==0 else 0 for item in mask_surface]
            mask            =	[i*j*k for i,j,k in zip(mask, mask_surface, mask_surface2)]
            mask_y.append(mask)
        Result.append(mask_y)
    return Result

#For a surface containing several surface, this function selectes the most dense surface and creates a surface containing only this selected surface  
def SelectSurface(vscene, surf, surfType, groupContainer):
    vTimeIndex          =   0
    vAllStatistics 		= 	surf.GetStatistics()
    vNames	       		= 	vAllStatistics.mNames
    vValues        		= 	vAllStatistics.mValues
    volID	            = 	[a for a, x in enumerate(vNames) if x == "Volume"]
    vol		            =	[vValues[x] for x in volID]
    if surfType         ==  "Nucleolus":
        areaID	        = 	[a for a, x in enumerate(vNames) if x == "Area"]
        area		    =	[vValues[x] for x in areaID]
        volumeToAreaRatio         =   [x/y for x,y in zip(vol, area)]
        SelectedID      =   max(xrange(len(volumeToAreaRatio)), key=volumeToAreaRatio.__getitem__)
    else:
        SelectedID      =   max(xrange(len(vol)), key=vol.__getitem__)
    verts=surf.GetVertices(SelectedID) 
    vNormals			=	surf.GetNormals(SelectedID)  
    faces=surf.GetTriangles(SelectedID) 
    vNucleiSurface		=	vImaris.GetFactory().CreateSurfaces()
    vNucleiSurface.AddSurface(verts, faces, vNormals, vTimeIndex)
    vNucleiSurface.SetName(surfType)
    groupContainer.AddChild(vNucleiSurface, -1)
    vscene.AddChild(groupContainer, -1)
    return vNucleiSurface
  
#Creates a spots of XY, and Z radius in a given channel and returns the these spots   
def Segment_FISH_spots(channnelIndex, vDiameter_XY_index, vDiameter_Z_index, name, GroupContainer):
    vImage                  =   vImaris.GetDataSet()
    vScene                  =   vImaris.GetSurpassScene()
    aSubtractBackground		=	1
    aRegionsOfInterest      =   None
    #I set the minimum threshold for intensity max at 10% because this worked for the images that I processed manually so far
    aSpotFiltersString		=     '"Quality" above automatic threshold'
    aEstimateDiameterXYZ	=	[vDiameter_XY_index, vDiameter_XY_index, vDiameter_Z_index]
    vSpots				    =	vImaris.GetImageProcessing().DetectEllipticSpots(vImage,aRegionsOfInterest,channnelIndex,aEstimateDiameterXYZ,aSubtractBackground,aSpotFiltersString)
    vSpots.SetName(name)
    GroupContainer.AddChild(vSpots, -1)
    vScene.AddChild(GroupContainer, -1)
    return vSpots
    
#This function first creates a channel by selecting voxel contianing 50% of highest intensity values      
#If Type==spot, this function segments the new chanel into spots returns spots using previous function, if Type == surface, this function segments channel into surface, and returns the surface instance
def SegHighIntensity(chIndex, surf, ChannelName, ObjectName, numberIndex, Type, GroupContainer, ParametersList):
    vImage                  =   vImaris.GetDataSet()
    vScene                  =   vImaris.GetSurpassScene()
    res                     =   Get_Mask_data(surf, chIndex)
    result                  =   getHightIntensity(res[0], res[1])
    AddChannel(result, vImage, numberIndex) #CC channel
    vImage.SetChannelName(numberIndex-1,ChannelName)
    if Type                 ==  "spot":
        result              =   Segment_FISH_spots(numberIndex-1, ParametersList[7], ParametersList[8], ObjectName, GroupContainer)
    else: 
        result              =   Segment_Surface(vImage, numberIndex-1, ParametersList[5], ObjectName,ParametersList[6])#CC segmented
    return result

#==============================================================================
# Function required to create folder to save the plugins results
#==============================================================================
#Function to create a folder under the same directory as the images to save files that are produced
def CreateDirectoryToSaveFiles(Result_pathway):
    if os.path.exists(Result_pathway):
        tkMessageBox.showinfo(title="Alert", message="Please save the folder 'XTFISHInsideNucleus_Result' under another name first!")
        quit()
    else:
        os.makedirs(Result_pathway)

#==============================================================================
# Function required to concatenate files containing the same information into one table
#==============================================================================
def ConcatenatedTables(FileNameList, Result_pathway, FISHSpotNames):
    AllFiles                                =       os.listdir(Result_pathway) #get all files in the Result_pathway directory
    AllParamExported                        =       ["FISHChromocenterDistanceTable", "FISHPeripheryDistanceTable", "FISHNucleolusDistanceTable", "FISHPositionTable"]
    for parameter in AllParamExported:
        for FISHChannel in FISHSpotNames:
            FileType                        =       parameter   + "_" + FISHChannel
            AllFilesWithParam               =       [i for i in AllFiles if FileType in i]
            DataResult                          =       pd.DataFrame()
            for File in AllFilesWithParam:
                SelectedFile                    =       os.path.join(Result_pathway, File)
                data                            =       pd.read_csv(SelectedFile, header='infer',decimal='.')
                DataResult                      =       pd.concat([DataResult, data], axis=1)
                os.remove(SelectedFile)
            if len(DataResult)>0:
                if parameter    ==    "FISHPositionTable":
                    ColumnNames                     =       [a+b for a in FileNameList for b in list("XYZ")]
                    DataResult.columns              =       ColumnNames
                else:
                    DataResult.columns              =       FileNameList
                vPathToSaveTables                   =       os.path.join(Result_pathway, FileType)
                DataResult.to_csv(path_or_buf=vPathToSaveTables + ".csv", na_rep='', float_format=None, columns=None, header=True, index=False, decimal='.')

#==============================================================================
# These functions calculate the distances between the spots and the surfaces
#==============================================================================
#This function returns the volume value of surface
def GetVolume(surf):
    vAllStatistics 		= 	surf.GetStatistics()
    vNames	       		= 	vAllStatistics.mNames
    vValues        		= 	vAllStatistics.mValues
    VolumeId	        = 	[a for a, x in enumerate(vNames) if x == "Volume"]
    Volume		        =	float(vValues[VolumeId[0]])
    return Volume

#This function calculates distance between a dataframe of coordinates and a given spot and returns either the distance or the index in the dataframe of closest coordinates to the spot in question
def CalculateDistanceBNSurfaceAndSpot(vert1, spot):
    if type(spot[0]) is list :
        result=[min([sum([np.square(x[0]-y[0]), np.square(x[1]-y[1])], np.square(x[2]-y[2])) for x in vert1]) for y in spot]
    else  :
        result=min([sum([np.square(x[0]-spot[0]), np.square(x[1]-spot[1])], np.square(x[2]-spot[2])) for x in vert1])
    return result

#Using the previous function, this function calculates the distance between FISH spots and the closest chromocenter surface
def CalculateFISHChromocenterDistance(FISHtable, ChromocenterSurf):
    logtime('CalculateFISHChromocenterDistance start')
    DistanceToChromocenterList=[]
    VerticesOfAllChromocenters=[]
    NumberOfChromocenter=ChromocenterSurf.GetNumberOfSurfaces()
    for ClosestChromocenterID in range(NumberOfChromocenter):
        VerticesOfOnetChromocenter  =   ChromocenterSurf.GetVertices(ClosestChromocenterID)
        VerticesOfAllChromocenters  =  VerticesOfAllChromocenters  +   VerticesOfOnetChromocenter
    DistanceToChromocenterList=CalculateDistanceBNSurfaceAndSpot(VerticesOfAllChromocenters, FISHtable)
    logtime('CalculateFISHChromocenterDistance end')
    return DistanceToChromocenterList

def CalculateDistance(FISHSpotList, ChromocenterSurface, NucleusSurface, NucleolusSurface, NucleusSurfaceVolume, FISHSpotNames, FileIndex, Result_pathway, DistanceOptions):
    NumberOfFISHChannels = len(FISHSpotNames)
    # Create list that will contain distances calculate
    FISHChromocenterDistanceTable = [[]] * NumberOfFISHChannels
    FISHPositionTable = [[]] * NumberOfFISHChannels
    FISHPeripheryDistanceTable = [[]] * NumberOfFISHChannels
    FISHNucleolusDistanceTable = [[]] * NumberOfFISHChannels
    count             =     0
    logtime('Calculating distances START - image_'+str(FileIndex))
    for FISHSpot in FISHSpotList:
        PositionTable =     FISHSpot.GetPositionsXYZ()
        # 		Step1: Calculate distances
        # ==============================================================================
        if DistanceOptions[2] and ChromocenterSurface is not None:
            ###Distance1: Calculate distance between FISH spot's center of mass and the closest chromocenter ###
            FISHToChromocenterDistance= CalculateFISHChromocenterDistance(PositionTable, ChromocenterSurface)
            FISHToChromocenterDistance_Normalised = [x/NucleusSurfaceVolume for x in FISHToChromocenterDistance]
            FISHChromocenterDistanceTable[count] =  FISHToChromocenterDistance_Normalised
            logtime('Chromocenter to FISH signal distance calculated - channelId: ' + FISHSpotNames[count]+' - Image '+str(FileIndex))
        if DistanceOptions[0] and NucleusSurface is not None:
            ###Distance2: Calculate FISH spot's center of mass to nucleus periphery distance###
            PeripheryVertices      =   NucleusSurface.GetVertices(0)
            FISHPeripheryDistance= CalculateDistanceBNSurfaceAndSpot(PeripheryVertices, PositionTable)
            FISHPeripheryDistance_Normalised = [x/NucleusSurfaceVolume for x in FISHPeripheryDistance]
            FISHPeripheryDistanceTable[count] = FISHPeripheryDistance_Normalised
            logtime('Nucleus periphery to FISH signal distance calculated - channelId: ' + FISHSpotNames[count]+' - Image '+str(FileIndex))
        if DistanceOptions[1] and NucleolusSurface is not None:
            ###Distance3: Calculate FISH spot's center of mass to nucleolus periphery distance###
            NucleolusVertices      =   NucleolusSurface.GetVertices(0)
            FISHNucleolusDistance= CalculateDistanceBNSurfaceAndSpot(NucleolusVertices, PositionTable)
            FISHNucleolusDistance_Normalised = [x/NucleusSurfaceVolume for x in FISHNucleolusDistance]
            FISHNucleolusDistanceTable[count] =     FISHNucleolusDistance_Normalised
            logtime('Nucleolus periphery to FISH signal distance calculated - channelId: ' + FISHSpotNames[count]+' - Image '+str(FileIndex))
        FISHPositionTable[count]=PositionTable
        count           +=  1
    logtime('Calculating distances END - image_'+str(FileIndex))
    resultList=[FISHChromocenterDistanceTable, FISHPeripheryDistanceTable, FISHNucleolusDistanceTable, FISHPositionTable]
    TableName = ["FISHChromocenterDistanceTable", "FISHPeripheryDistanceTable", "FISHNucleolusDistanceTable", "FISHPositionTable"]
    # 		Step2: Save dataframes containing results
    # ==============================================================================
    for param in range(len(resultList)):
        for channel_index in range(len(FISHSpotNames)):
            if len(resultList[param][channel_index])>0:
                SaveTable   =       pd.DataFrame(resultList[param][channel_index])
                vPathToSaveTables = os.path.join(Result_pathway, TableName[param])
                SaveTable.to_csv(path_or_buf=vPathToSaveTables +"_" + FISHSpotNames[channel_index] +"_"+str(FileIndex)+".csv", na_rep='', float_format=None, columns=None, header=True, index=False, decimal='.')


#==============================================================================
# This function:
# Segments chanenls into surface (nucleus, nucleolus and chromocenters) or spots (FISH signals),
# Calculates distances between spots and surfaces
# And saves results
#==============================================================================
def GetImageFeatures(FileIndex, Result_pathway, vFileName, DoSegmentation, DistanceOptions, ParametersList, BatchProcessing):
    global vImaris
    global numberIndex
    global SelectedChanelIndex, DAPIChannel
    global FileNameList
    vImage              =   	vImaris.GetDataSet()
    if vImage is not None:
        FileNameList.append(vFileName)
        numberIndex         = vImage.GetSizeC()
        if FileIndex    == 1 and DoSegmentation:
            SelectedChanelIndex, DAPIChannel= Ask_user(numberIndex)
        ChromocenterSurface =   NucleusSurface  =   NucleolusSurface    =   None
        date			=	str(datetime.datetime.now()).split(" ")[0]
        date			+=	" 00:00:00"
        vImage.SetTimePoint(0, date) # time point needs to be set for each image to avoid creating different time frames when closing and opening a new image.
        vImage                     =    vImaris.GetDataSet()
        if DoSegmentation:
            logtime('Segmentation START - image_'+str(FileIndex))
            vScene                     =    vImaris.GetSurpassScene()
            vFactory		           =	vImaris.GetFactory().CreateFactory()
            GroupOfObjects	           =	vFactory.CreateDataContainer()
            GroupOfObjects.SetName('Segmented objects')
            numberIndex     +=1
            if DistanceOptions[1]:
                SurfNP=Segment_Surface(vImage, DAPIChannel, ParametersList[0], "Detailed Nucleus segmentation",0)
                SurfNP2=Segment_Surface(vImage, DAPIChannel, ParametersList[1], "Rough Nucleus segmentation",0)
                res=Get_Mask_data(SurfNP, DAPIChannel)
                res2=Get_Mask_data(SurfNP2, DAPIChannel)
                resNuc=getLowIntensity(res[0], res[1], res2[0])
                AddChannel(resNuc, vImage, numberIndex)
                vImage.SetChannelName(numberIndex-1,"Low DAPI intensity")
                NucR=Segment_Surface(vImage, numberIndex-1, ParametersList[2], "Rough Nucleolus segmentation",ParametersList[3])
                numberIndex+=1
                logtime('Nucleolus surface segmented - Image '+str(FileIndex))
                NucleolusSurface=SelectSurface(vScene, NucR, "Nucleolus", GroupOfObjects) #Nucleolus is segmented
            SurfP=Segment_Surface(vImage, DAPIChannel, ParametersList[4], "Rougher Nucleus segmentation",0)
            NucleusSurface=SelectSurface(vScene, SurfP, "Nucleus", GroupOfObjects) #Nucleus is segmented
            logtime('Nucleus surface segmented - Image '+str(FileIndex))
            if DistanceOptions[2]:
                ChromocenterSurface          =  SegHighIntensity(DAPIChannel, SurfP,"High DAPI intensity",
                                                                 "Chromocenters",
                                                                 numberIndex, "surface", GroupOfObjects, ParametersList)#Chromocenters are segmented
                GroupOfObjects.AddChild(ChromocenterSurface, -1)
                vScene.AddChild(GroupOfObjects, -1)
                logtime('Chromocenter surfaces segmented - Image '+str(FileIndex))
                numberIndex     +=  1
            FISHSpotList               =    []
            FISHSpotNames              =    []
            for channelId in SelectedChanelIndex :
                FISHSpot           =    SegHighIntensity(channelId, SurfP,"High intensity FISH_Ch"+str(
                    channelId), "FISH_Spots_Ch"+str(
                    channelId), numberIndex, "spot", GroupOfObjects, ParametersList)#FISH signals are segmented
                logtime('FISH signal segemented - channelId: ' + str(channelId)+' - Image '+str(FileIndex))
                numberIndex     +=  1
                PositionTable =     FISHSpot.GetPositionsXYZ()
                if PositionTable is not None:
                    FISHSpotList       =   FISHSpotList + [FISHSpot]
                    FISHSpotNames      =   FISHSpotNames    +   ["FISH_Ch"+str(channelId)]
            logtime('Segmentation END - image_'+str(FileIndex))
        else:
            NucleusSurface, NucleolusSurface, ChromocenterSurface, FISHSpotNames, FISHSpotList  = GetSegmentedObjects(DistanceOptions, FileIndex)
        # if DistanceOptions[0] == (NucleusSurface is not None) and DistanceOptions[1] == (NucleolusSurface is not None) and DistanceOptions[2] == (ChromocenterSurface is not None):
        NucleusSurfaceVolume    =   GetVolume(NucleusSurface)
        CalculateDistance(FISHSpotList, ChromocenterSurface, NucleusSurface, NucleolusSurface, NucleusSurfaceVolume, FISHSpotNames, FileIndex, Result_pathway, DistanceOptions)
        if DoSegmentation:
            vPath = os.path.join(Result_pathway, vFileName+".ims")
            vImaris.FileSave(vPath, "")
            if BatchProcessing:
                vScene.RemoveChild(GroupOfObjects)
    return vImage is None, FISHSpotNames

#==============================================================================
# Function required to get segmented objects
#==============================================================================
def GetSegmentedObjects(DistanceOptions,FileIndex):
    logtime('Object detection START - image_'+str(FileIndex))
    NucleusSurface  =   NucleolusSurface    =   ChromocenterSurface =   None
    vScene                     =    vImaris.GetSurpassScene()
    vFactory		           =    vImaris.GetFactory()
    numberSceneInstance	       =	vScene.GetNumberOfChildren()
    ContainerNotFound          =    True
    i                          =    0
    while i <= numberSceneInstance and ContainerNotFound:
        selection 	           =	vScene.GetChild(i)
        vContainer             =	vFactory.ToDataContainer(selection)
        if vContainer is not None:
            ContainerNotFound      =    False
        i=i+1
    ContainerName             =    vContainer.GetName()
    if vContainer is not None and ContainerName=="Segmented objects":
        numberSceneInstance  =    vContainer.GetNumberOfChildren()
        i                    =    0
        FISHSpotNames        =    []
        FISHSpotList         =    []
        while i <= numberSceneInstance :
            selection 	          =    vContainer.GetChild(i)
            vObject               =    vFactory.ToSurfaces(selection)
            if vObject is not None:
                if vObject.GetName() == "Nucleus" and DistanceOptions[0]        :   NucleusSurface      = vObject
                if vObject.GetName() == "Nucleolus" and DistanceOptions[1]      :   NucleolusSurface    = vObject
                if vObject.GetName() == "Chromocenters" and DistanceOptions[2]  :   ChromocenterSurface = vObject
            vObject               =    vFactory.ToSpots(selection)
            if vObject is not None:
                PositionTable =     vObject.GetPositionsXYZ()
                if "FISH" in vObject.GetName() and PositionTable is not None:
                    FISHSpotNames     = FISHSpotNames   +    [vObject.GetName()]
                    FISHSpotList      = FISHSpotList    +    [vObject]
            i+=1
        logtime('Object detection END - image_'+str(FileIndex))
    return NucleusSurface, NucleolusSurface, ChromocenterSurface, FISHSpotNames, FISHSpotList
#==============================================================================
# Pop-up windows to ask user to set folder pathway and channels to segment
#==============================================================================
class Checkbar(Frame):
  def __init__(self, parent=None, picks=[], side=LEFT, anchor=W):
    Frame.__init__(self, parent)
    self.vars = []
    for pick in picks:
      var = IntVar()
      chk = Checkbutton(self, text=pick, variable=var)
      chk.pack(side=side, anchor=anchor, expand=YES)
      self.vars.append(var)
  def state(self):
    return map((lambda var: var.get()), self.vars)

def allstates(): 
    global User_selection
    global root
    if sum(list(lng.state()))>0:
        User_selection=list(lng.state())
        root.destroy()
    else:
        Message="Please select one of the options."
        Label(root, text=Message).grid(row=1)
 
def PopUpMessage(OPTIONS, Messge):
  global root
  global lng
  root = tk.Tk()
  label_text	=	Messge
  option		=	OPTIONS
  Label(root, text=label_text).grid(row=0)
  lng = Checkbar(root, option)
  lng.grid(row=2)
  lng.config(relief=GROOVE, bd=2)
  Button(root, text='Quit', fg="red", command=quit).grid(row=4)
  Button(root, text='Submit', fg="darkgreen", command=allstates).grid(row=5)
  root.mainloop()

#This function asks user to set some parameters FISH and DAPI channels.
def Ask_user(numberIndex):
    OPTIONS                         =       range(numberIndex)
    Messge                          =       "Please select the FISH signal channels: \n You can choose multiple channels."
    PopUpMessage(OPTIONS, Messge)
    SelectedChanelIndex	            =	    [i for i, x in enumerate(User_selection) if x == 1]
    Messge="Please select the DAPI channel: \n Please only choose one channel."
    PopUpMessage(OPTIONS, Messge)
    DAPIChannel	=	[i for i, x in enumerate(User_selection) if x == 1][0]
    return SelectedChanelIndex, DAPIChannel

def FinalTaskOverPopUp(FileIndex):
    tkMessageBox.showinfo(title="IMARIS::XTFISHInsideNucleus => Task completed", message="Task is completed!"
    +"\n Total number of images analysed "+str(FileIndex-1)
    +"\n"+str('*--*'*15)
    +"\n Please find in the folder XTFISHInsideNucleus_Result, images created after segmentation and the following tables (each column in table is the result of one image):"
    +"\n -  FISHChromocenterDistanceTable_Ch.csv, contains FISH to chromocenter distances;"
    +"\n -  FISHPositionTable_Ch.csv, contains FISH to spots center of mass coordinates;"
    +"\n -  FISHPeripheryDistanceTable_Ch.csv, contains FISH to nucleus periphery distances;"
    +"\n -  FISHNucleolusDistanceTable_Ch.csv, contains FISH to nucleolus distances;"
    +"\n"+str('*--*'*15)
    +"\n Notice: The segmentations and the subsequent distance calculation is as good as the quality of image. If after review of images the segmentation of one image isn't accurate remove the column of that image from table.")

#==============================================================================
# Function to get parameters for this plugin from file:XTFISHInsideNucleus_Parmaeters.csv
#==============================================================================
def GetPluginParameters():
    currentDirectory                         =       os.getcwd()
    AllFilesInDirectory                      =       os.listdir(currentDirectory)
    ParametersList                           =       None
    if "XTFISHInsideNucleus_Parmaeters.csv" in AllFilesInDirectory:
        ParameterData                            =       pd.read_csv("XTFISHInsideNucleus_Parmaeters.csv", sep=";", header='infer',decimal='.')
        if "Value" in ParameterData.columns:
            ParametersList                           =       list(ParameterData["Value"])
        else:
            tkMessageBox.showinfo(title="Error", message="Please make sure the 'XTFISHInsideNucleus_Parmaeters.csv' file contains a column 'Value' containing the values necessary for this plugin.")
            quit()
    else:
        tkMessageBox.showinfo(title="Error", message="Please make sure there is a 'XTFISHInsideNucleus_Parmaeters.csv' in the folder containing the 'XTFISHInsideNucleus.py'.")
        quit()
    return ParametersList

#==============================================================================
# Main function:
# Connects to Imaris and get image
# Process images
#==============================================================================
def XTFISHInsideNucleus(aImarisId):
    logging.basicConfig(level=logging.DEBUG, filename= "log[XTFISHInsideNucleus.py].log") # If there is any error running the script, the error is logged in this file
    try:
        #Declare global variables
        global gLasttime
        global vImaris
        global FileNameList
        gLasttime               =    None
        logtime('Extension XTFISHInsideNucleus START')
        FileNameList            =   []
        #            Step1: Connect to Imaris and open image files and the analysis
        #==============================================================================
        vImarisLib			=	ImarisLib.ImarisLib()
        # Get an imaris object with id aImarisId
        vImaris             =   vImarisLib.GetApplication(aImarisId)
        ParametersList      =   GetPluginParameters()
        # """BEGIN LOOP OVER ALL IMAGES"""
        # Open File and get filename
        if vImaris is not None :
            logtime('Connected to Imaris')
            ListOfOptions                   =       [["Batch of images", "Just one image"], ["Segmentation & Distance estimation", "Distance estimation"], ["Nucleus", "Nucleolus", "Chromocenters"]]
            ListOfMessages                  =       ["Do you wish to run the script on a batch of images or just on one image already opened?", "Do you wish to do segmentation and distance estimation or just distance estimation?", "Which distances are you calculating? \n You can choose more than one option.\n FISH spot to:"]
            UserParameterList               =       []
            for i in range(len(ListOfOptions)):
                OPTIONS                         =       ListOfOptions[i]
                Messge                          =       ListOfMessages[i]
                PopUpMessage(OPTIONS, Messge)
                UserParameterList               =       UserParameterList + [User_selection]
            BatchProcessing	                =	    UserParameterList[0][0]
            DoSegmentation	                =	    UserParameterList[1][0]
            DistanceOptions                 =       UserParameterList[2]
            FileIndex                       =       1 #This  variable is used to count the number of files analysed
            if BatchProcessing  :
                # 		Step2: Here the user is asked to set the path to the folder containing image to be analysed
                #==============================================================================
                root1				    =	        Tkinter.Tk()
                Image_folder			=	        tkFileDialog.askdirectory(parent=root1, initialdir="/",title='Please select the directory containing the images to be processed. \n The folder containing the resulting files will be saved in this directory.')
                root1.destroy()
                Result_pathway          =           os.path.join(Image_folder, "XTFISHInsideNucleus_Result")
                CreateDirectoryToSaveFiles(Result_pathway)
                AllFilesInDirectory     =           os.listdir(Image_folder) #get all files in the Image_folder directory
                logtime('Get all files')
                AllFilesToBeProcessed   =           [i for i in AllFilesInDirectory if i.endswith('.ims') or i.endswith('.ics')] #select files with .ims or .ics extensions
                TotalNumberFile         =           len(AllFilesToBeProcessed)
                logtime('Start bach processing')
                if TotalNumberFile  >   0:
                    for vFileName in AllFilesToBeProcessed:
                        vFullFileName   =           os.path.join(Image_folder, vFileName)
                        vImaris.FileOpen(vFullFileName, "")
                        ImageIsEmpty, FISHSpotNames    =           GetImageFeatures(FileIndex, Result_pathway, vFileName, DoSegmentation, DistanceOptions, ParametersList, BatchProcessing)
                        if not ImageIsEmpty :
                            FileIndex	               +=  		1
                else:
                    tkMessageBox.showinfo(title="Alert", message="There is no .ims or .ics file detected in the selected folder.")
                    quit()
            else    :
                TotalNumberFile         =   1
                vFileName               =   vImaris.GetCurrentFileName()
                ImageIsEmpty            =   True
                if vFileName !=""       :
                    vFileName           =   vImaris.GetCurrentFileName()
                    vFilePath           =   os.path.dirname(vFileName)
                    Result_pathway      =   os.path.join(vFilePath, "XTFISHInsideNucleus_Result")
                    CreateDirectoryToSaveFiles(Result_pathway)
                    vFileName           =   os.path.split(vFileName)[1]
                    ImageIsEmpty, FISHSpotNames        =   GetImageFeatures(FileIndex, Result_pathway, vFileName, DoSegmentation, DistanceOptions, ParametersList, BatchProcessing)
                    FileIndex		+=		1
                if vFileName =="" or ImageIsEmpty :
                    tkMessageBox.showinfo(title="Alert", message="No image is detected. \n Please open an image and select on 'FISHInsideNucleus' again.")
                    quit()
            if FileIndex>1:
                logtime('Files saved and organised')
                ConcatenatedTables(FileNameList, Result_pathway, FISHSpotNames)
                logtime('XTFISHInsideNucleus extension done')
                FinalTaskOverPopUp(FileIndex)
        else:
            tkMessageBox.showinfo(title="Alert", message="Imaris application is not found!")
            logtime('Extension XTFISHInsideNucleus END')
    except:
        logging.exception("Oops:")
#==============================================================================
# End of extension    
#==============================================================================
